open Camlp4
open PreCast
open Printf
 
module Id = struct
  let name = "pa_exp"
  let version = "1.0"
end

module Make (Syntax : Sig.Camlp4Syntax) = struct
  open Sig
  include Syntax

  let rtyuio = Gram.Entry.mk "rtyuio"

  let rec convert_type = function
  | <:ctyp< $lid:i$ >> when i = "int" -> Common.Int
  | <:ctyp< $lid:i$ >> when i = "string" -> Common.String
  | <:ctyp< $lid:i$ >> when i = "float" -> Common.Float
  | <:ctyp< $lid:i$ >> when i = "char" -> Common.Char
  | <:ctyp< $lid:i$ >> when i = "bool" -> Common.Bool
  | <:ctyp< $t1$ $t2$ >> ->
    let t = convert_type t2 in
    (match t1 with
	| <:ctyp< $lid:i$ >> when i = "option" -> Common.Option t
	| <:ctyp< $lid:i$ >> when i = "list" -> Common.List t
	| _ -> failwith "only option and list allowed for type application")
  | _ -> failwith "this case is not representable using this AST"

  (* Grammar definition *)
  EXTEND Gram
  
    rtyuio: [
  		[ t = ctyp; EOI -> convert_type t ]
  	];

  END
  
  let my_parse_type str =
    print_endline (Common.string_of_type (Gram.parse rtyuio (Loc.mk str) (Stream.of_string str)))

  let _ =
    Common.parse_type := my_parse_type
end
  
let _=
  let module M = Register.OCamlSyntaxExtension (Id) (Make) in ()